<?php

/**
*
*	Stats:Extract
*	@author Nighthawk/Nekomata [#116310] | irfandahir.com
*	@version 0.6
*	@copyright Copyright (c) 2015-2016, Nighthawk
*
*/

/**
*	Class: StatsExtract (Main Class)
*	Methods:
*		[Public]
*			@method powered_by() @return string
*			@method parse_htmlstats(string $file) @return array
*			@method parse_userstats(string $file) @return array
*			@method parse_live_server_info(string $ip,int $port {,int $timeout}) array
*			@method parse_live_server_players(string $ip,int $port {,int $timeout}) array
*			@method float_fix(float $float, int $round) int|float
*			@method average(array $arr) 
*			@method peak(array $arr) 
*			@method sum(array $arr) 
*			@method relative_tos(int $s) 
*			@method calc_kpd(int $usgn) 
*			@method get_avatar_by_usgn(int $usgn) @return string
*			@method get_name_by_usgn(int $usgn) @return string
*/
class StatsExtract {

	//** Configuration **//
	//*******************//
	const VERSION = 0.6;
	const LOG = true;
	const LOG_PATH = "log.se";
	const DEFAULT_SOCKET_TIMEOUT = 5;
	//*******************//

	private $log = null;

/**
*	If LOG is true then opens the log path. It's done in __construct() because more efficiency
*/
	public function __construct(){
		if (self::LOG) {
			if (file_exists(self::LOG_PATH) && is_writable(self::LOG_PATH)) {
				$this->log = fopen(self::LOG_PATH, "a+"); 
			}
		}
	}
/**
*	__construct(), On loading, calls the simple_html_dom library.
*	If LOG is true then closes the log path. 
*/
	public function __destruct(){
		if (self::LOG) {
			if (file_exists(self::LOG_PATH)) {
				fclose($this->log);
			}
		}
	}

/**
*	Parses the serverstats.html to extract the user ranks, server information & graph values.
*	@param string $file_name path to serverstats.html for extraction
*	in an array; @return array(
*			['stats']=>array(
*				int ['uptime'],
*				float ['upload'],
*				float ['download'],
*				float ['total'],
*				int ['ranked-players']
*			),
*			['graph'] => array(
*				['today'|'yesterday']=>array(
*				[0]=>array(
*					float ['upload'],
*					float ['download'],
*					int ['players']
*				),
*				[n]=>array(
*					float ['upload'],
*					float ['download'],
*					int ['players']
*				),
*				[23]=>array(
*					float ['upload'],
*					float ['download'],
*					int ['players']
*				),
*			)
*		)
*	@example $svstats = $this->parse_htmlstats("path/to/serverstats.html")
*	@return array
*	@access public
*/
	public function parse_htmlstats($file_name) {
		if (!is_null($file_name)) {
			if (file_exists($file_name) && is_readable($file_name) && preg_match("/[Hh][Tt][Mm][Ll]/", basename($file_name))) {
				$tStart = microtime();
				$lines = file($file_name);
				if(!$lines){
					self::log("Error: [".__FUNCTION__."] Error loading \"".$file_name."\"");
					return false;
				}
				self::log("Debug: [".__FUNCTION__."] \"".$file_name."\" loaded. Parsing...");
				$htmlData = array(
					'stats' => array(),
					'graph' => array(
						'today' => array(),
						'yesterday' => array()
						)
					);
				$todayGraphFound = false;
				$todayGraphFilter = array();
				$yesterdayGraphFound = false;
				$yesterdayGraphFilter = array();
				foreach ($lines as $lineNo => $line) {
					if (strpos($line, "<meta name=\"generator\" content=\"Unreal Software's Counter-Strike 2D ")) {
						$filter = false;
						preg_match("/<meta name=\"generator\" content=\"Unreal Software's Counter-Strike 2D (.*)\">/", $line, $filter);
						//$htmlData['stats']['version'] = substr($line, 68, -2);
						$htmlData['stats']['version'] = $filter[1];
					}
					elseif (strpos($line, "<div class=\"sec\">Overview</div>")) {
						$uptimeFilter = array();
						$uploadFilter = array();
						$downloadFilter = array();
						$trafficFilter = array();
						$playerFilter = array();
						preg_match("/<span class=\"descr\">Uptime: <\/span> (.*)<br>/", $lines[$lineNo+1], $uptimeFilter);
						preg_match("/<span class=\"descr\">Upload Traffic: <\/span> (.*)<br>/", $lines[$lineNo+2], $uploadFilter);
						preg_match("/<span class=\"descr\">Download Traffic: <\/span> (.*)<br>/", $lines[$lineNo+3], $downloadFilter);
						preg_match("/<span class=\"descr\">Total Traffic: <\/span> (.*)<br>/", $lines[$lineNo+4], $trafficFilter);
						preg_match("/<span class=\"descr\">Ranked U.S.G.N. Users: <\/span> (.*)<br>/", $lines[$lineNo+5], $playerFilter);
						$htmlData['stats']['uptime'] = $uptimeFilter[1];
						$htmlData['stats']['upload'] = $uploadFilter[1];
						$htmlData['stats']['download'] = $downloadFilter[1];
						$htmlData['stats']['total'] = $trafficFilter[1];
						$htmlData['stats']['ranked-players'] = (int) $playerFilter[1];
					}
					elseif (strpos($line, "<div class=\"sec\">Traffic Today ")) {
						$todayGraphFound = true;
					}
					elseif ($todayGraphFound) {
						if (preg_match("/<td><a href=\"#\" title=\"Hour 00: /", $line)) {
							$todayGraphFound = false;
							for ($i=0; $i <= 23; $i++) {
								$todayGraphFilter[$i] = array();
								preg_match("/<td><a href=\"#\" title=\"Hour (.*): up (.*) mb, down (.*) mb\"><b>[0-9]{1,}<\/b><\/a><br><span class=\"ulc\">[0-9]{1,}<\/span><br><span class=\"dlc\">[0-9]{1,}<\/span><br><span class=\"plc\">(.*)<\/span><\/td>/", $lines[$lineNo+$i], $todayGraphFilter[$i]);
								$htmlData['graph']['today'][$i] = array(
									'upload' => $todayGraphFilter[$i][2],
									'download' => $todayGraphFilter[$i][3],
									'players' => $todayGraphFilter[$i][4],
								);
							}
						}
					}
					elseif (strpos($line, "<div class=\"sec\">Traffic Yesterday ")) {
						$yesterdayGraphFound = true;
					}
					elseif ($yesterdayGraphFound) {
						if (preg_match("/<td><a href=\"#\" title=\"Hour 00: /", $line)) {
							$yesterdayGraphFound = false;
							for ($i=0; $i <= 23; $i++) {
								$yesterdayGraphFilter[$i] = array();
								preg_match("/<td><a href=\"#\" title=\"Hour (.*): up (.*) mb, down (.*) mb\"><b>[0-9]{1,}<\/b><\/a><br><span class=\"ulc\">[0-9]{1,}<\/span><br><span class=\"dlc\">[0-9]{1,}<\/span><br><span class=\"plc\">(.*)<\/span><\/td>/", $lines[$lineNo+$i], $yesterdayGraphFilter[$i]);
								$htmlData['graph']['yesterday'][$i] = array(
									'upload' => $yesterdayGraphFilter[$i][2],
									'download' => $yesterdayGraphFilter[$i][3],
									'players' => $yesterdayGraphFilter[$i][4],
								);
							}
						}
					}
				}
				$tEnd = microtime();
				$timeTaken = number_format($tEnd - $tStart, 7);
				self::log("Debug: [".__FUNCTION__."] Parse complete in ".$timeTaken."s");
				return $htmlData;
			} else {
				self::log("Error: [".__FUNCTION__."] File \"".$file_name."\" does not exist, is invalid an file type or is not readable");
				return false;
			}
		} else {
			$this->log("Error: [".__FUNCTION__."] No file name given");
			return false;
		}
	}

/**
*	Parses the userstats.dat to extract user ranks and data.
*	Credits go to MikuAuahDark for the decoding script!
* 	@param string $file_name path to userstats.dat file
*	@param string $orderBy "rank" returns array by rank number & "usgn" returns by USGN (both highest score to lowest)
*	@return array(
*		[n]=>array(
*			string ['name'], //name used when first joined server
*			int ['usgn'],
*			int ['score'],
*			int ['frags'], //kills
*			int ['deaths'],
*			int ['tos'] //time on server in seconds
*			),
*		)
*	@return array(
*		[usgn]=>array(
*			string ['name'], //name used when first joined server
*			int ['score'],
*			int ['frags'], //kills
*			int ['deaths'],
*			int ['tos'] //time on server in seconds
*			),
*		)
*	@example $user = $this->parse_userstats("path/to/userstats.dat") //extraction by rank by default
*	@example $user = $this->parse_userstats("path/to/userstats.dat", "rank")
*	@example $user = $this->parse_userstats("path/to/userstats.dat", "usgn")
*	@access public
*/
	public function parse_userstats($file_name, $orderBy="rank") {
		if (!is_null($file_name)) {
			if (file_exists($file_name) && is_readable($file_name) && preg_match("/[Dd][Aa][Tt]/", basename($file_name))) {
				$tStart = microtime();
				$file = fopen($file_name, "rb");
				if(!$file) {
					self::log("Error: [".__FUNCTION__."] Error loading \"".$file_name."\"");
					return false;
				}
				self::log("Debug: [".__FUNCTION__."] \"".$file_name."\" loaded. Parsing...");
				fseek($file, 0, SEEK_END);
				$size = ftell($file);
				fseek($file, 17, SEEK_SET);
				$userData = array();
				if (preg_match("/[Uu][Ss][Gg][Nn]/", $orderBy)) {
					$index = 0;
					while (ftell($file) != $size) {
						$index++;
						$name = trim(fgets($file));
          				$userData[(int) self::decode($file)] = array(
          					"rank" => $index,
          					"name" => $name,
          					"score" => (int) self::decode($file),
          					"frags" => (int) self::decode($file),
          					"deaths" => (int) self::decode($file),
          					"tos" => (int) self::decode($file),
          					);
					}
				}
				elseif (preg_match("/[Rr][Aa][Nn][Kk]/", $orderBy)) {
					$index = 0;
					while (ftell($file) != $size) {
						$index++;
						$userData[$index] = array(
							"rank" => $index,
							"name" => trim(fgets($file)),
							"usgn" => (int) self::decode($file),
							"score" => (int) self::decode($file),
							"frags" => (int) self::decode($file),
							"deaths" => (int) self::decode($file),
							"tos" => (int) self::decode($file),
						);
					}
				}
				fclose($file);
				$tEnd = microtime();
				$timeTaken = number_format($tEnd - $tStart, 7);
				self::log("Debug: [".__FUNCTION__."] Parse complete in ".$timeTaken."s");
				return $userData;
			} else {
				self::log("Error: [".__FUNCTION__."] File \"".$file_name."\" does not exist, is invalid an file type or is not readable");
				return false;
			}
		} else {
			$this->log("Error: [".__FUNCTION__."] No file name given");
			return false;
		}
	}
/**
 * Socket handler
 * @see StatsExtract->parse_live_server_info()
 * @see StatsExtract->parse_live_server_players()
 */
private $socket_read;

/**
*	parse_live_server_info() connects to a server that's online on it's respective port to get server details. Such as the ones displayed in CS2D's serverlist.
*	Credits go to DC for sharing his script for this!
*	@param array parse_live_server_info() parse_live_server_info(string $ip, int $port {, int $timeout}) Returns server stats via socket connection
*	in an array; @return array(
*		string ['name'],
*		string ['map'],
*		int ['players'],
*		int ['max-players'],
*		int ['bots'],
*		string ['total-players'],
*		bool ['password'],
*		bool ['usgn-only'],
*		bool ['fow'],
*		bool ['ff'],
*		bool ['lua'],
*		string ['game-mode']
*		)
*	@example $svinfo = $this->parse_live_server_info(192.168.0.1,25235) With self::DEFAULT_SOCKET_TIMEOUT seconds timeout
*	@example $svinfo = $this->parse_live_server_info(192.168.0.1,25235,20) With 20 seconds timeout
*	@return array
*	@access public
*/
	public function parse_live_server_info($ip, $port, $timeout = self::DEFAULT_SOCKET_TIMEOUT){
		if(empty($ip)){
			$this->log("Error: [".__FUNCTION__."] No IP Address Specified");
			return false;
		}else{
			if(!filter_var($ip, FILTER_VALIDATE_IP)){
				$this->log("Error: [".__FUNCTION__."] Invalid IP Address (".$ip.")");
				return false;
			}
		}
		if(empty($port)){
			$this->log("Error: [".__FUNCTION__."] No Port Specified");
			return false;
		}else{
			if(is_integer($port)){
				if($port<1 || $port>65535){
					$this->log("Error: [".__FUNCTION__."] Invalid Port");
				}
			}else{
				$this->log("Error: [".__FUNCTION__."] Invalid Port");
				return false;
			}
		}
		$timeout = (int)$timeout;
		if($timeout<1) { $timeout=self::DEFAULT_SOCKET_TIMEOUT; }
		$this->log("Debug: [".__FUNCTION__."] Connecting to (".$ip.":".$port.")");
		$fp = @fsockopen("udp://".$ip.":".$port);
		if($fp){
			$this->log("Debug: [".__FUNCTION__."] Connected to (".$ip.":".$port.")");
			$string = chr(1).chr(0).chr(251).chr(1);
			stream_set_timeout($fp, $timeout);
			fwrite($fp, $string);
			$this->socket_read = fread($fp, 256);
			$this->log("Debug: [".__FUNCTION__."] Fetching data");
			$meta_data = stream_get_meta_data($fp);
			if($meta_data['timed_out'] || (ord(substr($this->socket_read, 0, 1)) == 0)){
				$this->log("Error: [".__FUNCTION__."] Connection timed out (".$ip.":".$port.")");
				fclose($fp);
				return false;
			}
			$b0 = $this->readByte();
			$b1 = $this->readByte();
			$b2 = $this->readByte();
			$b3 = $this->readByte();
			if($b0==1 && $b1==0 && $b2=251 && $b3==1){
				$this->log("Debug: [".__FUNCTION__."] Parsing data");
			    $mode = $this->readByte();
			   	$name = $this->readString();
			    $map = $this->readString();
			    $players = min($this->readByte(),32);
			    $players_max = min($this->readByte(),32);
       			$state = $this->readByte();
			    $gmi = ($mode & 32) ? $this->readByte() : 0;
			    $bots = $this->readByte();
			    $svinfo = array();
			    $svinfo['name'] = (string)$this->utf8_htmlentities($name);
			    $svinfo['map'] = (string)$map;
			    $svinfo['players'] = (int)$players;
			    $svinfo['max-players'] = (int)$players_max;
			    $svinfo['bots'] = (int)$bots;
			    $svinfo['password'] = ($mode & 1) ? true : false;
			    $svinfo['usgn-only'] = ($mode & 2) ? true : false;
			    $svinfo['fow'] = ($mode & 4) ? true : false;
			    $svinfo['ff'] = ($mode & 8) ? true : false;
			    $svinfo['lua'] = ($mode & 64) ? true : false;
			    switch ($gmi){
			    	case 0: $svinfo['game-mode'] = 'Standard'; break;
			    	case 1: $svinfo['game-mode'] = 'Deathmatch'; break;
			    	case 2: $svinfo['game-mode'] = 'Team Deathmatch'; break;
			    	case 3: $svinfo['game-mode'] = 'Construction'; break;
			    	case 4: $svinfo['game-mode'] = 'Zombies'; break;
			    	default: $svinfo['game-mode']= 'Unknown ('.$gmi.')';
			    }
			    $this->log("Debug: [".__FUNCTION__."] Parse complete; Disconnecting");
			    fclose($fp);
			    return $svinfo;
			}else{
				$this->log("Error: [".__FUNCTION__."] Unexpected server reply (',".$b0.",',',".$b1.",',',".$b2.",',',".$b3.",') - expected (1,0,251,1)");
				fclose($fp);
				return false;
			}
		}else{
			$this->log("Error: [".__FUNCTION__."] Failed to connect (".$ip.":".$port.")");
			return false;
		}
		$this->socket_read = null;
	}

/**
*	parse_live_server_players() connects to a server that's online on it's respective port to get server players.
*	Credits go to Kemine for allowing me to use his script for this
*	@param array parse_live_server_players() parse_live_server_players(string $ip, int $port {, int $timeout}) Returns server players via socket connection
*	in an array; @return array(
*		[int player id] = array(
			string ['name'],
			int ['score'],
			int ['deaths'],
			string ['team']
*		);
*	@example $svplayers = $this->parse_live_server_players(192.168.0.1,25235) With self::DEFAULT_SOCKET_TIMEOUT seconds timeout
*	@example $svplayers = $this->parse_live_server_players(192.168.0.1,25235,20) With 20 seconds timeout
*	@return array
*	@access public
*/
	public function parse_live_server_players($ip, $port, $timeout = self::DEFAULT_SOCKET_TIMEOUT) {
		if(empty($ip)){
			$this->log("Error: [".__FUNCTION__."] No IP Address Specified");
			return false;
		}else{
			if(!filter_var($ip, FILTER_VALIDATE_IP)){
				$this->log("Error: [".__FUNCTION__."] Invalid IP Address (".$ip.")");
				return false;
			}
		}
		if(empty($port)){
			$this->log("Error: [".__FUNCTION__."] No Port Specified");
			return false;
		}else{
			if(is_integer($port)){
				if($port<1 || $port>65535){
					$this->log("Error: [".__FUNCTION__."] Invalid Port");
				}
			}else{
				$this->log("Error: [".__FUNCTION__."] Invalid Port");
				return false;
			}
		}
		$timeout = (int)$timeout;
		if($timeout<1) { $timeout=self::DEFAULT_SOCKET_TIMEOUT; }
		$this->log("Debug: [".__FUNCTION__."] Connecting to (".$ip.":".$port.")");
		$fp = @fsockopen("udp://".$ip.":".$port);
		if($fp){
			$this->log("Debug: [".__FUNCTION__."] Connected to (".$ip.":".$port.")");
			$string = chr(1).chr(0).chr(251).chr(5);
			stream_set_timeout($fp, $timeout);
			fwrite($fp, $string);
			$this->socket_read = fread($fp, 2048);
			$this->log("Debug: [".__FUNCTION__."] Fetching player list");
			$meta_data = stream_get_meta_data($fp);
			if($meta_data['timed_out'] || (ord(substr($this->socket_read, 0, 1)) == 0)){
				$this->log("Error: [".__FUNCTION__."] Connection timed out (".$ip.":".$port.")");
				fclose($fp);
				return false;
			}
				$svplayers = array();
				$v_temp = substr(self::strToHex($this->socket_read), 10);
				$last = 0;
				for ($i = 1; $i <= 32; $i++) {
					$v_temp = substr($v_temp,$last);
					$userlen = hexdec(substr($v_temp,2,2))*2;
					if (hexdec(substr($v_temp,0,2))==0)	{
						break;
					}
					$id = hexdec(substr($v_temp,0,2));
					$svplayers[$id] = array(
						"name" => utf8_encode(self::hexToStr(substr($v_temp,4,$userlen))),
						"score" => hexdec(substr($v_temp,$userlen+10,2).substr($v_temp,$userlen+8,2).substr($v_temp,$userlen+6,2)),
						"deaths" => hexdec(substr($v_temp,$userlen+18,2).substr($v_temp,$userlen+16,2).substr($v_temp,$userlen+14,2)),
						);
					$svplayers[$id]["team"] = (hexdec(substr($v_temp,$userlen+4,2))==1 || hexdec(substr($v_temp,$userlen+4,2))==2) ? (hexdec(substr($v_temp,$userlen+4,2))==1) ? "Terrorist" : "Counter-Terrorist" : "Spectator";
					$last=$userlen+22;
				}
			    $this->log("Debug: [".__FUNCTION__."] Parse complete; Disconnecting");
			    fclose($fp);
			    ksort($svplayers);
			    return $svplayers;
		}else{
			$this->log("Error: [".__FUNCTION__."] Failed to connect (".$ip.":".$port.")");
			return false;
		}
		$this->socket_read = null;
	}

	public function powered_by() {
		echo "Powered by <a target=\"_blank\" title=\"Running Stats:Extract ".self::VERSION."\" href=\"http://www.unrealsoftware.de/files_show.php?file=16081\">Stats:Extract</a>";
	}

/**
 * Utility Methods
 */

	/**
	 * Round off a decimal
	 * @param float $float decimal for rounding
	 * @param int $round significant figure for rounding
	 */
	public function float_fix($float, $round) {
		return round(floatval($float),$round);
	}

	/**
	 * Calculate the average for graph values
	 * @param array $arr download, upload, players graph plot arrays
	 * @return int|float
	 */
	public function average($arr) {

		$sum = 0;
		foreach ($arr as $key => $value) {
			$sum = $sum + $value;
		}
		return $sum/count($arr);
	}

	/**
	 * Calculate the peak for graph values
	 * @param array $arr download, upload, players graph plot arrays
	 * @return int|float
	 */
	public function peak($arr) {
		$peak = 0;
		foreach ($arr as $key => $value) {
			if ($value > $peak) {
				$peak = $value;
			}
		}
		return $peak;
	}

	/**
	 * Calculate the sum of the graph values
	 * @param array $arr download, upload, players graph plot arrays
	 * @return int|float
	 */
	public function sum($arr) {
		$sum = 0;
		foreach ($arr as $key => $value) {
			$sum += $value; 
		}
		return $sum;
	}

	/**
	 * Change time on server format from seconds to relative
	 * @param int $s seconds
	 * @return string
	 * Source: http://stackoverflow.com/a/10668410
	 */
	public function relative_tos($seconds){
	    $ret = "";
	    $days = intval(intval($seconds) / (3600*24));
	    if($days> 0)
	    {
	        $ret .= $days."d ";
	    }
	    $hours = (intval($seconds) / 3600) % 24;
	    if($hours > 0)
	    {
	        $ret .= $hours."h ";
	    }
	    $minutes = (intval($seconds) / 60) % 60;
	    if($minutes > 0)
	    {
	        $ret .= $minutes."m ";
	    }
	    $seconds = intval($seconds) % 60;
	    if ($seconds > 0) {
	        $ret .= $seconds."s";
	    }
	    return $ret;
	}

	/**
	 * Calculate the KpD of a player
	 * @param int $frags
	 * @param int $deaths
	 * @return int|float
	 */
	public function calc_kpd($frags,$deaths){if($frags>0 && $deaths>0){return $this->float_fix($frags/$deaths,2);}elseif($deaths == 0){if($frags>0){return $frags;}elseif($frags==0){return floatval(0.00);}}elseif($frags == 0){return floatval(0.00);}}

/**
 * Get user's avatar by USGN by using UnrealSoftware's API
 *
 * @param int $usgn USGN to use
 */
	public function get_avatar_by_usgn($usgn) {
		$avatar = file_get_contents("http://www.unrealsoftware.de/getuserdata.php?id=".((int)trim($usgn))."&data=avatar");
		return (!empty($avatar) ? "http://www.unrealsoftware.de/".$avatar : false);
	}

/**
 * Get user's name by USGN by using UnrealSoftware's API
 *
 * @param int $usgn USGN to use
 */
	public function get_name_by_usgn($usgn) {
		$name = file_get_contents("http://www.unrealsoftware.de/getuserdata.php?id=".((int)trim($usgn))."&data=name");
		return (!empty($name) ? $name : false);
	}

	/**
	 * Private methods for @see StatsExtract->parse_live_server_info()
	 * Private methods for @see StatsExtract->parse_live_server_players()
	 */
	private function utf8_htmlentities($t,$q=ENT_COMPAT,$char='UTF-8',$denc=TRUE) {
		return htmlentities($t,$q,$char,$denc);
	}

	private function readString() {
		$length = ord(substr($this->socket_read,0,1));
		$string = substr($this->socket_read,1,$length);
		$this->socket_read = substr($this->socket_read,$length+1);
		return $string;
	}

	private function readByte() {
		$byte = ord(substr($this->socket_read,0,1)); 
		$this->socket_read = substr($this->socket_read,1);
		return $byte;
	}

	private function decode($file) {
		return ord(fread($file,1))+ord(fread($file,1))*256+ord(fread($file,1))*65536+ord(fread($file,1))*16777216;
	}

	private function strToHex($string) {
	    $hex='';
	    for ($i=0; $i < strlen($string); $i++) {
			if (strlen(dechex(ord($string[$i])))==1) {
				$hex .= "0".dechex(ord($string[$i]));
			} else {
				$hex .= dechex(ord($string[$i]));
			}
	    }
	    return $hex;
	}

	private function hexToStr($hex) {
	    $string='';
	    for ($i=0; $i < strlen($hex)-1; $i+=2)
	    {
	        $string .= chr(hexdec($hex[$i].$hex[$i+1]));
	    }
	    return $string;
	}

	/**
	*	@param string $string to log
	*	@method mixed log() log(string $string) logs error/debugs
	*	@access private
	*/
	private function log($string) {
		if (self::LOG) {
			if ($this->log) {
				fwrite($this->log, "[".date("d-m-Y H.i.s") . substr((string)microtime(), 1, 8)."]".$string."\n");
			}
		}
	}

}